package ca.grimoire.maven.taglib;

import java.io.IOException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import ca.grimoire.maven.ArtifactDescription;
import ca.grimoire.maven.NoArtifactException;

/**
 * Writes version information about the specified maven artifact to the page.
 * The version information is drawn from either the thread context class loader
 * (if available there) or the handler's own class loader (if available there).
 * <p>
 * The groupId and artifactId for the maven artifact must both be provided via
 * the {@link #setGroupId(String)} and {@link #setArtifactId(String)} methods
 * prior to the first call to {@link #doStartTag()}.
 */
public class VersionTag extends TagSupport {

    private ArtifactDescription artifact;
    private String artifactId;
    private String groupId;

    /**
     * Prints the located version information to the page context, then returns
     * {@link javax.servlet.jsp.tagext.Tag#EVAL_PAGE}.
     * 
     * @return EVAL_PAGE.
     * @throws JspException
     *             if there is a problem writing to the page output stream.
     */
    @Override
    public int doEndTag() throws JspException {
        try {
            pageContext.getOut().print(artifact.getVersion());
            return EVAL_PAGE;

        } catch (IOException ioe) {
            throw new JspException(ioe);
        }
    }

    /**
     * Locates the version for the group and artifact specified. If either is
     * <code>null</code>, this throws an exception. Always returns
     * {@link javax.servlet.jsp.tagext.Tag#SKIP_BODY} otherwise.
     * 
     * @throws IllegalStateException
     *             if the groupId or artifactId have not been set.
     * @return SKIP_BODY.
     * @throws JspException
     *             if the configured groupId and artifactId do not exist on
     *             either the thread context classloader or the tag handler's
     *             classloader.
     */
    @Override
    public int doStartTag() throws JspException {
        if (groupId == null)
            throw new IllegalStateException("groupId has not been set");
        if (artifactId == null)
            throw new IllegalStateException("artifactId has not been set");

        try {
            getArtifactVersion();
        } catch (NoArtifactException nae) {
            throw new JspException(nae);
        }

        return SKIP_BODY;
    }

    /**
     * Locates an {@link ArtifactDescription} for a given group and artifact
     * identifier.
     * 
     * @param groupId
     *            the group identifier of the artifact to find.
     * @param artifactId
     *            the artifact identifier for the artifact.
     * @return the located artifact information.
     * @throws NoArtifactException
     *             if no artifact information is available.
     */
    public ArtifactDescription getVersionInfo(String groupId, String artifactId)
            throws NoArtifactException {
        try {
            return ArtifactDescription.locate(groupId, artifactId, pageContext
                    .getServletContext());
        } catch (NoArtifactException nae) {
            return ArtifactDescription.locate(groupId, artifactId);
        }
    }

    /**
     * Releases and resets the tag handler to its post-construction state.
     * Before any further tag lifecycle (except {@link #release()}) can be
     * called the groupId and artifactId properties must be set again.
     */
    @Override
    public void release() {
        super.release();

        groupId = artifactId = null;
    }

    /**
     * Configures the artifactId to print information for. The artifactId passed
     * must not be <code>null</code>.
     * 
     * @param artifactId
     *            the artifact identifier to locate.
     * @throws IllegalArgumentException
     *             if the artifactId is <code>null</code>.
     */
    public void setArtifactId(String artifactId) {
        if (artifactId == null)
            throw new IllegalArgumentException("artifactId");
        this.artifactId = artifactId;
    }

    /**
     * Configures the groupId to print information for. The groupId passed must
     * not be <code>null</code>.
     * 
     * @param groupId
     *            the artifact groupId to locate.
     * @throws IllegalArgumentException
     *             if the groupId is <code>null</code>.
     */
    public void setGroupId(String groupId) {
        if (groupId == null)
            throw new IllegalArgumentException("groupId");
        this.groupId = groupId;
    }

    private void getArtifactVersion() throws NoArtifactException {
        artifact = getVersionInfo(groupId, artifactId);
    }
}
